// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* DataSetStruct.h	& XMM Science Analysis System (SAS) 
* 
* Class representing abstract data type DataSetStruct
* 
* (c) 2000 ESA - Written by Mark Thomas, CARA
* 
* $Id: DataSetStruct.h,v 1.3 2000/05/24 14:22:18 mthomas Exp $
* 
*****************************************************************/ 
#ifndef DATA_SET_STRUCT_H
#define DATA_SET_STRUCT_H


#include <Dal.h>


class AttributableStruct;
class TableStruct;
class DataSetStruct;
class DataSetStructServer;


class LabelledStruct
{
	public :
		virtual ~LabelledStruct() {}

		virtual const string&		name() const = 0;
		virtual void 			name( const string& name ) = 0;
		virtual bool			nameFound() const = 0;
		virtual unsigned int		number() const = 0;

};


class AttributeStruct : public virtual LabelledStruct, public virtual Child<AttributableStruct>
{
	public :
		virtual ~AttributeStruct() {}

		virtual Attribute::DataType	dataType() const = 0;
		virtual int			intValue() const = 0;
		virtual double			realValue() const = 0;
		virtual string			stringValue() const = 0;
		virtual bool			boolValue() const = 0;
		virtual bool			dataTypeFound() const = 0;
		virtual bool			valueFound() const = 0;

};


class AttributableStruct : public virtual LabelledStruct
{
	public :
		virtual ~AttributableStruct() {}

		virtual unsigned long		attributeStructs() const = 0;
		virtual const AttributeStruct*	attributeStruct( unsigned int attributeNumber ) const = 0;

};


class ColumnStruct : public virtual AttributableStruct, public virtual Child<TableStruct>
{
	public :
		virtual ~ColumnStruct() {}

		virtual Column::DataType	dataType() const = 0;
		virtual bool			dataTypeFound() const = 0;

};


class BlockStruct : public virtual AttributableStruct, public virtual Child<DataSetStruct>
{
	public :
		virtual ~BlockStruct() {}
		virtual Block::Type 		type() const = 0;

};


class TableStruct : public virtual BlockStruct
{
	public :
		virtual ~TableStruct() {}

		virtual unsigned int		columnStructs() const = 0;
		virtual const ColumnStruct*	columnStruct( unsigned int columnNumber ) const = 0;
		virtual unsigned long		rows() const = 0;
		virtual bool			rowsFound() const = 0;

};


class ArrayStruct : public virtual BlockStruct
{
	public :
		virtual ~ArrayStruct() {}

		virtual Array::DataType		dataType() const = 0;
		virtual bool			dataTypeFound() const = 0;

};


class DataSetStruct : public virtual AttributableStruct, public virtual Child<DataSetStructServer> 
{
	public :
		virtual ~DataSetStruct() {}

		virtual unsigned int 		blockStructs() const = 0;
		virtual const 			BlockStruct* blockStruct( unsigned int blockNumber ) const = 0;
		virtual const 			ArrayStruct* arrayStruct( unsigned int blockNumber ) const = 0;
		virtual const 			TableStruct* tableStruct( unsigned int blockNumber ) const = 0;
		virtual void 			check() const = 0;
};


class DataSetStructServer
{
	public :
		virtual ~DataSetStructServer() {}

		virtual void 			readDataSetStructs( const string& fileName, bool stopIfCheckFails ) = 0;
		virtual void 			releaseDataSetStructs() = 0;

		virtual unsigned int 		dataSetStructs() const = 0;
		virtual const DataSetStruct*	dataSetStruct( unsigned int dataSetStructNumber ) const = 0;
		virtual DataSetStruct*		dataSetStruct( unsigned int dataSetStructNumber ) = 0;
		virtual const DataSetStruct*	readDataSet( const string& setName ) = 0;

};


extern DataSetStructServer* dataSetStructServer;


#endif

