// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dscp
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Copy an object
	Description: Copy an object


*/


#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <CompoundName.h>


#include "dscp.h"

void dscp()
{
	PROC
	const string		from( stringParameter( "from" ) );
	const string		to( stringParameter( "to" ) );
	const CompoundName	fcn( from, CompoundName::Any, 0, Dal::Modify );
	const CompoundName	tcn( to, CompoundName::Any, 0, Dal::Modify );
	//const bool		withPosition( booleanParameter( "withposition" ) );
	//int			position( intParameter( "position" ));

	switch( fcn.state() )
	{
		case CompoundName::DataSetName :
			// DataSet -> DataSet
			if( tcn.state() != CompoundName::DataSetName )
			{
				errstr << "The dataset with name '" << from << "' may only be copied to another dataset. The destination object with name '" << to << "' is not a dataset" << Sas::error( "incompatibleObjectTypes" );

			} // if

			if( fcn.dataSetName() == tcn.dataSetName() )
			{
				errstr << "The dataset with name '" << from << "' may not be copied to itself" << Sas::error( "cannotCopyToSelf" );
			} // if
			dataSetServer -> copy( from, to );

			break;

		case CompoundName::BlockName :

			if( tcn.state() == CompoundName::DataSetName )
			{
				// Block -> DataSet
				if( fcn.dataSetName() == tcn.dataSetName() )
				{
					errstr << "The block with name '" << from << "' may not be copied to itself" << Sas::error( "cannotCopyToSelf" );
				} // if
				tcn.dataSet() -> add( fcn.block() );
			}
			else if( tcn.state() == CompoundName::BlockName )
			{
				// Block -> Block
				if( fcn.qualifiedBlockName() == tcn.qualifiedBlockName() )
				{
					errstr << "The block with name '" << from << "' may not be copied to itself" << Sas::error( "cannotCopyToSelf" );
				} // if
				tcn.dataSet() -> add( fcn.block(), tcn.blockName() );
			}
			else
			{
				errstr << "The block with name '" << from << "' may not be copied to the destination object with name '" << to << "'" << Sas::error( "incompatibleObjectTypes" );
			} // if

			break;

		case CompoundName::ColumnName :

			if( tcn.state() == CompoundName::BlockName )
			{
				// Column -> Table
				if( fcn.qualifiedBlockName() == tcn.qualifiedBlockName() )
				{
					errstr << "The column with name '" << from << "' may not be copied to itself" << Sas::error( "cannotCopyToSelf" );
				} // if
				tcn.table() -> add( fcn.column() );
			}
			else if( tcn.state() == CompoundName::ColumnName )
			{
				// Column -> Column
				if( fcn.qualifiedColumnName() == tcn.qualifiedColumnName() )
				{
					errstr << "The column with name '" << from << "' may not be copied to itself" << Sas::error( "cannotCopyToSelf" );
				} // if
				tcn.table() -> add( fcn.column(), tcn.columnName() );
			}
			else
			{
				errstr << "The column with name '" << from << "' may not be copied to the destination object with name '" << to << "'" << Sas::error( "incompatibleObjectTypes" );
			} // if

			break;

		case CompoundName::DataSetAttributeName :
		case CompoundName::BlockAttributeName :
		case CompoundName::ColumnAttributeName :
		{
			if(
				tcn.state() == CompoundName::DataSetAttributeName
				|| tcn.state() == CompoundName::BlockAttributeName
				|| tcn.state() == CompoundName::ColumnAttributeName )
			{
				// Attribute -> Attribute
				Attribute* srcAttribute( fcn.attribute() );
				Attribute* dtnAttribute( tcn.attribute() );
				*srcAttribute = *dtnAttribute;
			}
			else if(
				tcn.state() == CompoundName::DataSetName
				|| tcn.state() == CompoundName::BlockName
				|| tcn.state() == CompoundName::ColumnName )
			{
				// Attribute -> Attributable
				Attribute* srcAttribute( fcn.attribute() );
				Attributable* dtnAttributable( tcn.attributable() );
				dtnAttributable -> addAttribute( srcAttribute );
			}
			else
			{
				errstr << "Cannot move attribute '" << from << "' to '" << to << "'" << Sas::error( "moveNotPossible" );
			} // if
		}
			break;
		default :
			errstr << "Object type '" << from << "' not supported" << Sas::error( "unsupportedObjectType" );
	} // switch

	ENDPROC
}

