// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dslatts
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: List the attributes in the given attributable objects
	Description: List the attributes in the given attributable objects


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <dal_utilities.h>
#include <CompoundName.h>


#include "dslatts.h"

static void
listAttributes( const Attributable* attributable, ostream& os )
{
	PROC

	os << "ATTRIBUTABLE: " << attributable -> qualifiedName() << endl;
	os << "NAME    ";
	os << "TYPE    ";
	os << "VALUE   ";
	os << "LABEL   ";
	os << "UNITS   ";
	os << endl;

	const unsigned int	attributes( attributable -> attributes() );

	for( unsigned int i = 0; i < attributes; ++i )
	{
		const Attribute*	attribute( attributable -> attribute( i ) );
		os << attribute -> name() << "   ";
		os << toString( attribute -> dataType() ) << "   ";
		os << attribute -> asString() << "   ";
		os << attribute -> label() << "   ";
		os << attribute -> units() << "   ";
		os << endl;
	} // for
	ENDPROC
}



void dslatts()
{
	PROC
	const vector<string>	objects( stringListParameter( "objects" ) );
	ostream&		os( cout );

	for( unsigned int i = 0; i < objects.size(); ++i )
	{
		const string		name( objects[ i ] );
		DataSet*		set( dataSetServer -> open( name, Dal::Read, DataSetServer::Low ) );
		const CompoundName	cn( name, CompoundName::AnyAttributableName, set );
		Attributable*		attributable( cn.attributable() );

		listAttributes( attributable, os );

		dataSetServer -> close( set );
	} // for
	ENDPROC
}


