// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dslcols
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: List the columns in the given tables and/or or datasets
	Description: List the columns in the given tables and/or or datasets


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <ExtDal.h>
#include <dal_utilities.h>
#include <CompoundName.h>



#include "dslcols.h"


static void
listColumns( const Table* table, ostream& os )
{
	PROC
	const unsigned int	columns = table -> columns();
	os << "ColumnNames   " << "Type        " << "Dims       " << "Units" << endl;
	for( unsigned int i = 0; i < columns; ++i )
	{
		const Column* column = table -> column( i );
		os << column -> name() << "   " << toString( column -> dataType() )
		<< "   ";
		if( column -> elements() > 1 || column -> dataType() == Dal::DString )
		{
			os << "[";
			for( unsigned int j = 0; j < column -> size().size(); ++j )
			{
				if( j > 0 )
				{
					os << ",";
				} // if
				os << column -> size()[ j ];
			} // for
			os << "]     ";
		}
		else
		{
			os << "scalar   ";
		} // if
		os << column -> units();
		os << endl;
	} // for
	ENDPROC
}


void dslcols()
{
	PROC
	const vector<string>	objects( stringListParameter( "objects" ) );
	ostream&		os( cout );

	for( unsigned int i = 0; i < objects.size(); ++i )
	{
		const string		name( objects[ i ] );
		DataSet*		set( dataSetServer -> open( name, Dal::Read, DataSetServer::Low ) );
		const CompoundName	cn( name, CompoundName::Any, set );

		switch( cn.state() )
		{
			case CompoundName::DataSetName :
			{
				TableIterator* ti( blockIteratorServer -> tableIterator( set ) );
				Table* table;
				while( (table = ti -> next()) )
				{
					os << "TABLE: " << table -> qualifiedName() << endl;
					listColumns( table, os );
				} // while
			}
				break;
			case CompoundName::TableName :
				listColumns( set -> table( name ), os );
				break;
			default :
				errstr << "Object type " << name << " not supported" << Sas::error( "unsupportedObjectType" );
		} // switch

		dataSetServer -> close( set );
	} // for
	ENDPROC
}
