// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dsrename
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Rename an object
	Description: Rename an object


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <CompoundName.h>


#include "dsrename.h"

void dsrename()
{
	PROC
	const string		name( stringParameter( "object" ) );
	const string		newName( stringParameter( "newname" ) );

	DataSet*		set( dataSetServer -> open( name, Dal::Modify, DataSetServer::Low ) );
	const CompoundName	cn( name, CompoundName::Any, set );

	switch( cn.state() )
	{
		case CompoundName::DataSetName :
			set -> rename( newName);
			break;
		case CompoundName::BlockName :
		case CompoundName::TableName :
		case CompoundName::ArrayName :
			set -> block( name ) -> rename( newName );
			break;
		case CompoundName::ColumnName :
			set -> table( name ) -> column( name ) -> rename( newName );
			break;
		case CompoundName::DataSetAttributeName :
		case CompoundName::BlockAttributeName :
		case CompoundName::ColumnAttributeName :
			cn.attributable() -> attribute( name ) -> rename( newName );
			break;
		default :
			errstr << "Object type " << name << " not supported" << Sas::error( "unsupportedObjectType" );
	} // switch

	dataSetServer -> close( set );

	ENDPROC
}
