// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dsreplacenulls
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Replace the null values in an object with a new value
	Description: Replace the null values in an object with a new value


*/

#include <Tracer.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <CompoundName.h>



#include "dsreplacenulls.h"


template<class T>
static void
replaceNulls( Data* d, T*(Data::*data)() const, unsigned long elements, T value )
{
	PROC
	T* theData = (d ->* data)();
	for( unsigned long i = 0; i < elements; ++i )
	{
		if( d -> isNull( i ) )
		{
			theData[ i ] = value;
		} // if
	} // for
	ENDPROC
}


static void
replaceNulls(
	Data*		data,
	unsigned long	elements,
	const string&	value )
{
	PROC
	switch( data -> dataType() )
	{
		case Dal::Int8 :
			replaceNulls<int8>( data, &Data::int8Data, elements, atoi( value.c_str() ) );
			break;
		case Dal::Uint16 :
			replaceNulls<uint16>( data, &Data::uint16Data, elements, atoi( value.c_str() ) );
			break;
		case Dal::Int16 :
			replaceNulls<int16>( data, &Data::int16Data, elements, atoi( value.c_str() ) );
			break;
		case Dal::Uint32 :
			replaceNulls<uint32>( data, &Data::uint32Data, elements, atoi( value.c_str() ) );
			break;
		case Dal::Int32 :
			replaceNulls<int32>( data, &Data::int32Data, elements, atoi( value.c_str() ) );
			break;
		case Dal::Real32 :
			replaceNulls<real32>( data, &Data::real32Data, elements, atof( value.c_str() ) );
			break;
		case Dal::Real64 :
			replaceNulls<real64>( data, &Data::real64Data, elements, atof( value.c_str() ) );
			break;
		default :
			errstr << "Data type not supported" << Sas::error( "unsupportedDataType" );

	} // switch
	ENDPROC
}


static void
replaceNulls(
	Column*		column,
	const string&	value )
{
	PROC
	if( column -> cellType() == Column::Fixed )
	{
		ColumnData*		columnData = column -> data();
		const unsigned long	elements = column -> rows() * column -> elements();

		replaceNulls( columnData, elements, value );

		delete columnData;
	}
	else
	{
		for( unsigned long i = 0; i < column -> rows(); ++i )
		{
			CellData*		cellData = column -> cellData( i );
			const unsigned long	elements = cellData -> elements();

			replaceNulls( cellData, elements, value );

			delete cellData;
		} // for
	} // if
	ENDPROC
}

 
void dsreplacenulls()
{
	PROC
	const vector<string>	objects( stringListParameter( "objects" ) );
	const string		value( stringParameter( "value" ) );

	for( unsigned int i = 0; i < objects.size(); ++i )
	{
		const string		name( objects[ i ] );
		const CompoundName	cn( name, CompoundName::Any, 0, Dal::Modify, DataSetServer::HighLow );

		switch( cn.state() )
		{
			case CompoundName::ArrayName :
			{
				Array*			array = cn.array();
				ArrayData*		arrayData = array -> data();
				const unsigned long	elements = array -> elements();

				replaceNulls( arrayData, elements, value );

				delete arrayData;
			}
				break;

			case CompoundName::ColumnName :
			{
				Column*			column = cn.column();

				replaceNulls( column, value );

			}
				break;
			default :
				errstr << "Object type " << name << " not supported" << Sas::error( "unsupportedObjectType" );
		} // switch
	} // for
	ENDPROC
}
