// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dssetattr
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Set/Add an attribute
	Description: Set/Add an attribute


*/

#include <Tracer.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <dal_utilities.h>
#include <CompoundName.h>

#include "dssetattr.h"

void dssetattr()
{
	PROC
	const string		to( stringParameter( "to" ) );
	const string		label( stringParameter( "label" ) );
	const string		units( stringParameter( "units" ) );
	DataSet*		set( dataSetServer -> open( to, Dal::Modify ) );
	const CompoundName	cn( to, CompoundName::AnyAttributeName, set );
	Attributable*		attributable( cn.attributable() );

	//if( set -> hasBlock( to ) )
	//{
	//	Block* block = set -> block( to );
	//	attributable = block;
	//	if( block -> type() == Block::TableT )
	//	{
	//		Table* table = set -> table( to );
	//		if( table -> hasColumn( to ) )
	//		{
	//			attributable = table -> column( to );
	//		} // if
	//	} // if
	//} // if

	Converter converter;

	Attribute* attribute = attributable -> addAttribute(
						to,
						label,
						units );

	*attribute = stringParameter( "value" );

	switch( converter.toAttributeDataType( stringParameter( "type" )))
	{
		case Attribute::Bool :
			*attribute = attribute -> asBool();
			break;
		case Attribute::Int :
			*attribute = attribute -> asInt();
			break;
		case Attribute::Real :
			*attribute = attribute -> asReal();
			break;
		case Attribute::DString :
			//attribute -> asString(); // Already a string.
			break;
		default :
			errstr << "Invalid switch" << Sas::error( "internalError" );
			break;
	} // switch

	dataSetServer -> close( set );

	ENDPROC
}
