// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dssetcolumnelement
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Set the value of a column element
	Description: Set the value of a column element


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <dal_utilities.h>

// *******************************************
// The following line should be uncommented if
// access to the DAL CompoundName class is
// required
// *******************************************
// #include <CompoundName.h>



#include "dssetcolumnelement.h"

template<class T>
static void
setElements(
	ColumnData*	cd,
	T* (Data::* data)() const,
	unsigned long	fromelement,
	unsigned long	elements,
	T		v )
{
	PROC

	T*	d;
	Data*	dd = cd;
	d = (dd ->*data)();
	//d = (cd ->*data)();

	for( unsigned long i = 0; i < cd -> count(); ++i )
	{
		for( unsigned long j = 0; j < elements; ++j )
		{
			d[ cd -> elements() * i + fromelement + j ] = v;
		} // for
	} // for

	ENDPROC
}


void dssetcolumnelement()
{
	PROC

	const vector<string>	columns( stringListParameter( "columns" ));
	const unsigned long	fromrow( intParameter( "fromrow" ));
	const unsigned long	fromelement( intParameter( "fromelement" ));
	const string		value( stringParameter( "value" ));

	for( unsigned int i = 0; i < columns.size(); ++i )
	{
		const string		columnName( columns[ i ] );
		DataSet*		set( dataSetServer -> open( columnName, Dal::Modify, DataSetServer::Low ) );
		Table*			table( set -> table( columnName ) );
		Column*			column( table -> column( columnName ) );

		const unsigned long	rows( intParameter( "rows" ) ?  intParameter( "rows" ) : column -> rows() - fromrow );
		const unsigned long	elements( intParameter( "elements" ) ?  intParameter( "elements" ) : column -> elements() - fromelement );
		ColumnData*		cd( column -> data( fromrow, rows ) );

		switch( column -> dataType() )
		{
			case Column::Int8 :
				setElements(
					cd,
					&Data::int8Data,
					fromelement,
					elements,
					int8( atoi( value.c_str() )));
				break;
			case Column::Uint16 :
				setElements(
					cd,
					&Data::uint16Data,
					fromelement,
					elements,
					uint16( atoi( value.c_str() )));
				break;
			case Column::Int16 :
				setElements(
					cd,
					&Data::int16Data,
					fromelement,
					elements,
					int16( atoi( value.c_str() )));
				break;
			case Column::Uint32 :
				setElements(
					cd,
					&Data::uint32Data,
					fromelement,
					elements,
					uint32( atoi( value.c_str() )));
				break;
			case Column::Int32 :
				setElements(
					cd,
					&Data::int32Data,
					fromelement,
					elements,
					atoi( value.c_str() ));
				break;
			case Column::Real32 :
				setElements(
					cd,
					&Data::real32Data,
					fromelement,
					elements,
					real32( atof( value.c_str() )));
				break;
			case Column::Real64 :
				setElements(
					cd,
					&Data::real64Data,
					fromelement,
					elements,
					atof( value.c_str() ));
				break;
			case Column::Bool :
			case Column::DString :
			default :
				errstr << "The column with qualified name '" << column -> qualifiedName() << "' has datatype '" << toString( column -> dataType() ) << "' which could not be handled" << Sas::error( "unhandledDataType" );
		} // switch

		delete cd;

		dataSetServer -> close( set );
	} // for

	ENDPROC
}
