// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dssetdata
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Copy an object's data to another object
	Description: Copy an object's data to another object. THe source and destination must be the same type of object


*/

#include <Tracer.h>
#include <errstr.h>
#include <param.h>
#include <dal_utilities.h>
#include <CompoundName.h>



#include "dssetdata.h"

void dssetdata()
{
	PROC
	const string	to( stringParameter( "to" ));
	const string	from( stringParameter( "from" ));

	CompoundName	tcn( to, CompoundName::Any, 0, Dal::Modify, DataSetServer::HighLow );
	CompoundName	fcn( from );

	if( tcn.state() != fcn.state() )
	{
		errstr << "The 'from' value '" << from << "' is compatible with the to' value '" << to << "'" << Sas::error( "incompatibleObjectTypes" );
	} // if

	if( fcn.state() == CompoundName::ArrayName )
	{
		errstr << "The setting of array data is not yet implemented" << Sas::error( "notYetImplemented" );
	} // if

	setData( tcn.column(), fcn.column() );

	ENDPROC
}
