// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dssetlabel
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Set the label of an array, table, column or attribute
	Description: Set the label of an array, table, column or attribute


*/

#include <Tracer.h>
//#include <errstr.h>
#include <param.h>

#include <Dal.h>

// #include <dal_utilities.h>

#include <CompoundName.h>



#include "dssetlabel.h"

void dssetlabel()
{
	PROC
	const string	name( stringParameter( "object" ) );
	const string	label( stringParameter( "label" ) );

	DataSet*		set( dataSetServer -> open( name, Dal::Read, DataSetServer::Low ) );
	const CompoundName	cn( name, CompoundName::Any, set );

	switch( cn.state() )
	{
		case CompoundName::ArrayName :
			set -> array( name ) -> relabel( label );
			break;

		case CompoundName::ColumnName :
			set -> table( name ) -> column( name ) -> relabel( label );
			break;
		case CompoundName::DataSetAttributeName :
			set -> attribute( name ) -> relabel( label );
			break;
		case CompoundName::BlockAttributeName :
			set -> block( name ) -> attribute( name ) -> relabel( label );
			break;
		case CompoundName::ColumnAttributeName :
			set -> table( name ) -> column( name ) -> attribute( name ) -> relabel( label );
			break;
		default :
			errstr << "Object type " << name << " not supported" << Sas::error( "unsupportedObjectType" );

		dataSetServer -> close( set );
	} // switch
	ENDPROC
}

