// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dsstats
	Package: daltools
	Author: Hassan Siddiqui
	Affiliation: Vega/ESA
	Email: hsiddiqu@astro.estec.esa.nl
	Purpose: Produce dataset statistics
	Description: Produce dataset statistics


*/

#include <Tracer.h>
// *******************************************
// The following line should be uncommented if
// access to the error handling interface is
// required
// *******************************************
#include <errstr.h>

// *******************************************
// The following line should be uncommented if
// access to the parameter interface is
// required
// *******************************************
#include <param.h>

// *******************************************
// The following line should be uncommented if
// access to the DAL is required
// *******************************************
#include <Dal.h>

// *******************************************
// The following line should be uncommented if
// access to the Extended DAL is required
// *******************************************
#include <ExtDal.h>

// *******************************************
// The following line should be uncommented if
// access to the DAL utilities library is
// required
// *******************************************
// #include <dal_utilities.h>

// *******************************************
// The following line should be uncommented if
// access to the DAL CompoundName class is
// required
// *******************************************
#include <CompoundName.h>

/* External classes */
#include "Statistic.h"

/* Task header */
#include "dsstats.h"

void dsstats()
{
	PROC;
	const vector<string>	objects( stringListParameter( "objects" ) );
	
	for( unsigned int i = 0; i < objects.size(); ++i )
	  {
	    const string	name( objects[ i ] );
	    DataSet *		set( dataSetServer -> open( name, Dal::Read ) );
	    const CompoundName	cn( name, CompoundName::Any, set );
	    
	    switch( cn.state() )
	      {
	      case CompoundName::DataSetName :
		{
		  TableIterator * ti( blockIteratorServer -> tableIterator( set ) );
		  Table * table;
		  while ( (table = ti -> next()) )
		    {
		      for (unsigned i = 0; i < table->columns(); ++i)
			{
			  Column * column = table->column(i);
			  Statistic statistic(column);
			  errstr
			    << statistic
			    << Sas::message(Msg::App, Msg::Always);
			}
		    } // while
		}
		  break;
		case CompoundName::TableName :
		  {
		    Table * table = set->table( name );
		    for (unsigned i = 0; i < table->columns(); ++i)
		      {
			Column * column = table->column(i);
			Statistic statistic(column);
			errstr 
			  << statistic
			  << Sas::message(Msg::App, Msg::Always);

		      }
		  }
		  break;
		case CompoundName::ColumnName :
		  {
		    Column * column = set->table(name)->column(name);
		    Statistic statistic(column);
		    errstr 
		      << statistic
		      << Sas::message(Msg::App, Msg::Always);
		    break;
		  }
		default :
		  errstr << "Object type " 
			 << name 
			 << " not supported" 
			 << Sas::error( "unsupportedObjectType" );
		} // switch
	    
	    dataSetServer -> close( set );
	  } // for

	ENDPROC;
}
