// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dsstruct
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Get the structure of a list of datasets
	Description: Get the structure of a list of datasets


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <dal_utilities.h>

#include "dsstruct.h"


static const unsigned int nrStdAttributes = 0;
static const string datasetDirective = "dataset";
static const string indent = "	";


static string
quote( const string& thisString )
{
	PROC
	RETURN string( "\"" ) + thisString + "\"";
}


//static string
//toString( Attribute::DataType	type )
//{
//	PROC
//	static string stringTypes[] = { "Bool", "Int", "Real", "String" };
//	RETURN stringTypes[ int( type ) - 1 ];
//}


//static string
//toString( Dal::DataType	type )
//{
//	PROC
//	static string stringTypes[] = { "Bool", "Int8", "Uint16", "Int16", "Uint32", "Int32", "Real32", "Real64", "String" };
//	RETURN stringTypes[ int( type ) - 1 ];
//}


void
dumpAttributes(
	const Attributable *	attributable,
	ostream&		os,
	const string&		in,
	unsigned int		firstNr )
{
	PROC
	for( unsigned int i = firstNr; i < attributable -> attributes(); ++i )
	{
		const Attribute * attribute = attributable -> attribute( i );
		os << in << "attribute" << endl;
		os << in << "<" << endl;
		os << in << indent << "name" << indent << quote( attribute -> name() ) << endl;
		os << in << indent << "type" << indent << toString( attribute -> dataType() ) << endl;
		os << in << indent << "value" << indent << quote( attribute -> asString() ) << endl;
		os << in << ">" << endl;
	} // for
	ENDPROC
}


static void
dumpTable(
	const Table*	table,
	ostream&	os )
{
	PROC
	os << indent << "table" << endl;
	os << indent << "<" << endl;
	os << indent << indent << "name" << indent << quote( table -> name() ) << endl;
	os << indent << indent << "rows" << indent << table -> rows() << endl;
	dumpAttributes( table, os, indent + indent, 0 );
	for( unsigned int i = 0; i < table -> columns(); ++i )
	{
		const Column* column = table -> column( i );
		os << indent << indent << "column" << endl;
		os << indent << indent << "<" << endl;
		os << indent << indent << indent << "name" << indent << quote( column -> name() ) << endl;
		os << indent << indent << indent << "type" << indent << toString( column -> dataType() ) << endl;
		dumpAttributes( column, os, indent + indent + indent, 0 );
		os << indent << indent << ">" << endl;
	} // for
	os << indent << ">" << endl;
	ENDPROC
}


static void
dumpArray(
	const Array*	array,
	ostream&	os )
{
	PROC
	os << indent << "array" << endl;
	os << indent << "<" << endl;
	os << indent << indent << "name" << indent << quote( array -> name() ) << endl;
	os << indent << indent << "type" << indent << toString( array -> dataType() ) << endl;
	dumpAttributes( array, os, indent + indent, 0 );
	os << indent << ">" << endl;
	ENDPROC
}


static void
dumpDataSet(
	const string&	setName,
	ostream&	os )
{

	PROC

	DataSet* set = dataSetServer -> open(
						setName,
						DataSet::Read,
						DataSetServer::HighLow );

	os << datasetDirective << endl;
	os << "<" << endl;
	os << indent << "name" << indent << quote( set -> name() ) << endl;

	dumpAttributes( set, os, indent, nrStdAttributes );

	for( unsigned int i = 0; i < set -> blocks(); ++i )
	{
		if( set -> block( i ) -> type() == Block::TableT )
			dumpTable( set -> table( i ), os );
		else
			dumpArray( set -> array( i ), os );
	} // for
	os << ">" << endl;
	dataSetServer -> close( set );

	ENDPROC
}


void
dsstruct()
{
	PROC
	ostream&		os = cout;
	vector<string>		setNames = stringListParameter( "sets" );

	for( unsigned int i = 0; i < setNames.size(); ++i )
	{
		dumpDataSet( setNames[ i ], os );
	} // for
	ENDPROC
}

