// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dstranstype
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Convert the datatype of a list of objects
	Description: Convert the datatype of a list of objects


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <dal_utilities.h>
#include <CompoundName.h>


#include "dstranstype.h"

void dstranstype()
{
	PROC
	const vector<string>	objects( stringListParameter( "objects" ) );
	Dal::DataType		type( toDalDataType( stringParameter( "type" )));
	
	for( unsigned int i = 0; i < objects.size(); ++i )
	{
		const string		name( objects[ i ] );
		DataSet*		set( dataSetServer -> open( name, Dal::Modify ) );
		const CompoundName	cn( name, CompoundName::Any, set );

		switch( cn.state() )
		{
			case CompoundName::ArrayName :
			{
				Array* array = set -> array( name );
				convertType( array, type );
				break;
			}
			case CompoundName::ColumnName :
			{
				Table* table = set -> table( name );
				Column* column = table -> column( name );
				convertType( column, type );
				break;
			}
			default :
				errstr << "Object type " << name << " not supported" << Sas::error( "unsupportedObjectType" );
		} // switch
		dataSetServer -> close( set );
	} // if
	ENDPROC
}
